import { Subject, Observable } from 'rxjs';
import { BucketDragDrop, IBucketContainerRef, IBucketItemRef, SelectionBoundary, BucketCloneModel, BucketDropBeforeParams, BucketGroupNameType } from './types';
/**
 *
 */
declare type BucketDropBeforeCallback = (params: BucketDropBeforeParams) => any;
declare type BucketDropBeforeEmitter = (params: {
    changeList: any[];
}) => any;
/**
 * 한개의 버킷 컨테이너.
 * - 동일 그룹명끼지 모델 전송이 가능하다.
 * - sender 역할의 컨테이너는 receiver 로 전달만 가능하고, 자체 소팅은 지원하지 않는다.
 * - receiver 역할의 컨테이너는 sender 로 부터 전달을 받을 수 있고, 자체 소팅을 지원한다.
 *
 * @export
 * @class JdBucketContainerRef
 * @implements {IBucketContainerRef}
 */
export declare class JdBucketContainerRef<TM = any> implements IBucketContainerRef<TM> {
    constructor();
    protected uid: string;
    protected itemRefMap: Map<string, IBucketItemRef>;
    protected bucketList: TM[];
    protected bucketGroupName: BucketGroupNameType;
    protected bucketMax: number;
    protected bucketIsReceiver: boolean;
    protected bucketIsMultiple: boolean;
    protected elDragContainer: Element | null;
    protected hookDropBefore: BucketDropBeforeCallback | null;
    protected dragItemRefs: IBucketItemRef[];
    protected subjectDropped: Subject<BucketDragDrop>;
    protected subjectChangeState: Subject<IBucketContainerRef>;
    protected lazyChangeStateTimer: any | null;
    protected lazyChangeStateDelay: number;
    CLONE_FLAG_KEY: string;
    CLONE_FLAG_VALUE: string;
    /**
     * 그룹명
     * @readonly
     * @type {BucketGroupNameType}
     */
    get groupName(): BucketGroupNameType;
    /**
     * 컨테이너에 담을 수 있는 최대 수
     * @readonly
     * @type {number}
     */
    get max(): number;
    /**
     * 해당 컨테이너가 receiver 인지 여부.
     * @readonly
     * @type {boolean}
     */
    get isReceiver(): boolean;
    /**
     * 멀티 dnd 지원 여부.
     * @readonly
     * @type {boolean}
     */
    get isMultiple(): boolean;
    /**
     * 현재 컨테이너가 포함한 모델 갯수가 최대 갯수에 도달했는지 여부.
     * @readonly
     * @type {boolean}
     */
    get isMaximum(): boolean;
    /**
     * draggable DOM
     * @readonly
     * @type {(Element | null)}
     */
    get elContainer(): Element | null;
    /**
     * 지정한 hookDropBefore 함수.
     * @readonly
     * @type {(BucketDropBeforeCallback | null)}
     */
    get onDropBefore(): BucketDropBeforeCallback | null;
    /**
     * 그룹명 지정
     * @param {BucketGroupNameType} groupName
     */
    setGroupName(groupName: BucketGroupNameType): void;
    /**
     * receiver 인지 지정
     * @param {boolean} is
     */
    setReceiver(is: boolean): void;
    /**
     * 멀티플 지원여부 지정
     * @param {boolean} is
     */
    setMultiple(is: boolean): void;
    /**
     * 버킷에 담을 수 있는 최대 수 지정
     * @param {number} [max=-1]
     */
    setMax(max?: number): void;
    /**
     * 모델 목록
     * @param {*} list
     */
    setList(list: any): void;
    /**
     * draggable DOM 참조 지정
     * @param {Element} element
     */
    setElContainer(element: Element): void;
    /**
     * 컨테이너 상태 변경 알림을 delay 만큼 느슨하게 하도록 지정.
     * @param {number} delay
     */
    setLazyStateChangeDelay(delay: number): void;
    /**
     * sender 컨테이너에서 receiver 컨테이너로 드랍을 하고,
     * list 를 merge 하기 직전 실행될 (validate, filter ... 를 직접 할) 함수 지정.
     * @param {BucketDropBeforeCallback} handle
     */
    setDropBefore(handle: BucketDropBeforeCallback): void;
    /**
     * 해당 컨테이너의 uid 반환
     * @returns {string}
     */
    getUid(): string;
    /**
     * 가지고 있는 모델 목록 반환
     */
    getList(): TM[];
    /**
     * 옵저버: 컨테이너 상태 변경
     * @returns {Observable<IBucketContainerRef>}
     */
    observeChangeState(): Observable<IBucketContainerRef>;
    /**
     * 옵저버: 컨테이너에 드랍
     * @returns {Observable<BucketDragDrop>}
     */
    observeDropped(): Observable<BucketDragDrop>;
    /**
     * 알림: 컨테이너 상태 변경
     * setLazyStateChangeDelay 가 지정되어 있다면 lazy 하게 알림, 그렇지 않다면 바로 알림.
     */
    dispatchChangeState(): void;
    /**
     * 알림: 컨테이너에 드랍
     * @param {BucketDragDrop} params
     */
    dispatchDropped(params: BucketDragDrop): void;
    /**
     * 드래깅 버킷 아이템 추가.
     * (드래깅 - 드래그 아이템으로 선택, 드래그 중인 것)
     * @param {IBucketItemRef} itemRef
     */
    addDragItem(itemRef: IBucketItemRef): void;
    /**
     * 지정된 boundary 영역에 hit 되는 버킷 아이템을 찾아서 추가
     * @param {SelectionBoundary} boundary
     */
    addDragItemToBoundary(boundary: SelectionBoundary): void;
    /**
     * 드래깅 버킷 아이템 제거
     * @param {IBucketItemRef} itemRef
     */
    removeDragItem(itemRef: IBucketItemRef): void;
    /**
     * 드래깅 버킷 아이템으로 지정
     * @param {IBucketItemRef} itemRef
     */
    setDragItem(itemRef: IBucketItemRef): void;
    /**
     * 드래깅 버킷 아이템 반환
     * @returns {IBucketItemRef[]}
     */
    getDragItems(): IBucketItemRef[];
    /**
     * 드래깅 버킷 아이템 모두 제거
     */
    flushDragItem(): void;
    /**
     * 드래깅으로 등록된 버킷 아이템 중 지정된 itemRef 와 일치하는것 index 반환.
     * @param {IBucketItemRef} itemRef
     * @returns {number}
     */
    findDragItemIndex(itemRef: IBucketItemRef): number;
    /**
     * list 중 지정된 itemRef 의 model 과 일치하는것 index 반환.
     * @param {IBucketItemRef} itemRef
     * @returns {number}
     */
    findItemRefIndex(itemRef: IBucketItemRef): number;
    /**
     * fromContainer 와 그룹명이 같은지 여부.
     * @param {JdBucketContainerRef} fromContainer
     * @returns {boolean}
     */
    isGroupSame(fromContainer: JdBucketContainerRef): boolean;
    /**
     * 해당 컨테이너가 드래깅 중이 아이템을 넣을 수 있느지 여부.
     * fromContainer 의 정보와 해당 컨테이너의 조건에 따라 드랍이 가능한지 여부를 판단함.
     * @param {JdBucketContainerRef} fromContainer
     * @returns {boolean}
     */
    isGroupInsertable(fromContainer: JdBucketContainerRef): boolean;
    /**
     * list 의 index 아이템을 제거.
     * @param {number} index
     */
    removeByIndex(index: number): void;
    /**
     * list 중 model 을 찾아서 동일한 model 의 아이템을 제거.
     * @param {*} model
     */
    removeByModel(model: any): void;
    /**
     * element 로 버킷 아이템을 찾아서 반환
     * @param {Element} element
     * @returns {IBucketItemRef}
     */
    findItemRefByElement(element: Element): IBucketItemRef;
    /**
     * 버킷 아이템 등록.
     * @param {IBucketItemRef} itemRef
     */
    joinItemRef(itemRef: IBucketItemRef): void;
    /**
     * 버킷 아이템 등록 해제.
     * @param {IBucketItemRef} itemRef
     */
    unjoinItemRef(itemRef: IBucketItemRef): void;
    /**
     * 등록된 버킷 아이템 맵.
     * @returns {Map<string, IBucketItemRef>}
     */
    getItemRefs(): Map<string, IBucketItemRef>;
    /**
     * draggable clone 시 치환할 모델.
     * draggable 에서 드랍을 하고 모델이 합쳐지기 직전 onClone 이 발생되는데, 이 때 이동될 모델을 치환할 수 있다.
     * onClone 시 버킷쪽에서 멀티플 선택한 모델 목록으로 합쳐버려도 되나,
     * dropBefore 를 통해 직접 값을 filter(예: 중복 id 는 넣지 않음, 어떤 type 은 제외하고 넣음 등) 하는 기능을 지원하려면,
     * onClone 시 플래그 모델로 치환 시키고 onDrop 타이밍에 플래그 모델을 제거,
     * dropBefore 의 유무/응답에 따라 버킷쪽에서 선택된 모델이나, dropBefore 에서 정제된 목록으로 합친다.
     * @returns {BucketCloneModel}
     */
    createCloneModel(): BucketCloneModel;
    /**
     * fromContainer 의 드래깅 중인(드랍한) 아이템을 insertIndex 에 현재 리스트에 합친다.
     * - dropBefore 가 지정된 경우 사용자 응답을 기다리고, 응답이 배열인 경우는 응답한 배열로 합친다.
     *   그렇지 않은 경우 응답값이 true 에 해당하는 경우만 드랍한 아이템으로 합친다.
     *   그 외는 무시하고 합치지 않는다.
     * fnEmitter 콜백의 경우 UI 단에서는 dropBefore 에 따라 선/후행 업데이트를 해야할 수 있어서
     * list 가 변경된 상태를 UI 단에서 알아야하는 경우 콜백을 해준다.
     * @param {number} insertIndex
     * @param {IBucketContainerRef} fromContainer
     * @param {BucketDropBeforeEmitter} [fnEmitter]
     */
    mergeToDrop(insertIndex: number, fromContainer: IBucketContainerRef, fnEmitter?: BucketDropBeforeEmitter): Promise<void>;
    /**
     * merge 할 때 현재 가지고 있는 목록을 정제하여 반환.
     * @protected
     * @returns {TM[]}
     */
    protected getMergeCurrentList(): TM[];
    /**
     * merge 할 때 합쳐져야할 대상 목록을 정제하여 반환.
     * @protected
     * @param {TM[]} addList
     * @returns {TM[]}
     */
    protected getMergeInsertList(addList: TM[]): TM[];
    /**
     * 파기
     */
    destroy(): void;
}
export {};
